"""Constanst for Win32API"""
# flake8: noqa
# spaces and more spaces

WM_CREATE         = 0x0001
WM_DESTROY        = 0x0002
WM_MOVE           = 0x0003
WM_SIZE           = 0x0005
WM_ACTIVATE       = 0x0006
WM_SETFOCUS       = 0x0007
WM_KILLFOCUS      = 0x0008
WM_CLOSE          = 0x0010
WM_ERASEBKGND     = 0x0014
WM_SYSCOLORCHANGE = 0x0015
WM_ENABLE         = 0x000A
WM_CHILDACTIVATE  = 0x0022
WM_PAINT          = 0x000F
WM_GETMINMAXINFO  = 0x0024
WM_GETHOTKEY      = 0x0033
WM_SETHOTKEY      = 0x0050
WM_NCCALCSIZE     = 0x0083
WM_NCHITTEST      = 0x0084
WM_NCPAINT        = 0x0085
WM_NCACTIVATE     = 0x0086
WM_NCLBUTTONDOWN  = 0x00A1
WM_NCLBUTTONUP    = 0x00A2
WM_INITDIALOG     = 0x0110
WM_COMMAND        = 0x0111
WM_SIZING         = 0x0214
WM_MOVING         = 0x0216
WM_SYSCOMMAND     = 0x0112
WM_LBUTTONUP      = 0x0202
WM_MOUSELEAVE     = 0x02A3


HTNOWHERE, HTCLIENT, HTCAPTION = (0, 1, 2)
HWND_TOP, HWND_BOTTOM, HWND_TOPMOST, HWND_NOTOPMOST = 0, 1, -1, -2
HTLEFT, HTRIGHT, HTTOP, HTTOPLEFT, HTTOPRIGHT, HTBOTTOM, HTBOTTOMLEFT, HTBOTTOMRIGHT = range(10, 18)

WMSZ_LEFT        = 1
WMSZ_RIGHT       = 2
WMSZ_TOP         = 3
WMSZ_TOPLEFT     = 4
WMSZ_TOPRIGHT    = 5
WMSZ_BOTTOM      = 6
WMSZ_BOTTOMLEFT  = 7
WMSZ_BOTTOMRIGHT = 8

LWA_COLORKEY, LWA_ALPHA = 1, 2
WHITE_BRUSH, BLACK_BRUSH, COLOR_WINDOW = 0, 4, 5
GWL_WNDPROC, GWL_STYLE, GWL_EXSTYLE = -4, -16, -20
IDI_APPLICATION, LR_LOADFROMFILE, LR_DEFAULTSIZE, IMAGE_ICON  = 32512, 0x10, 0x0040, 1
SW_HIDE, SW_NORMAL, SW_MINIMIZE, SW_MAXIMIZE, SW_SHOW, SW_RESTORE, SW_SHOWDEFAULT = 0, 1, 2, 3, 5, 9, 10
EXPAND_BORDER, TOPEXTENDWIDTH, BOTTOMEXTENDWIDTH, LEFTEXTENDWIDTH, RIGHTEXTENDWIDTH = 8, 8, 8, 8, 8
SC_MOVE, SC_MINIMIZE, SC_MAXIMIZE, SC_MOUSEMENU, SC_RESTORE, SC_DEFAULT = 0xF010, 0xF020, 0xF030, 0xF090, 0xF120, 0xF160

WS_OVERLAPPED       = 0x00000000
WS_THICKFRAME       = 0x00040000
WS_MAXIMIZEBOX      = 0x00010000
WS_MINIMIZEBOX      = 0x00020000
WS_SYSMENU          = 0x00080000
WS_DLGFRAME         = 0x00400000
WS_BORDER           = 0x00800000
WS_CAPTION          = 0x00C00000
WS_OVERLAPPEDWINDOW = 0x00CF0000
WS_CLIPCHILDREN     = 0x02000000
WS_CLIPSIBLINGS     = 0x04000000
WS_VISIBLE          = 0x10000000
WS_DISABLED         = 0x08000000
WS_CHILDWINDOW      = 0x40000000
WS_CHILD            = 0x40000000
WS_POPUP            = 0x80000000
WS_POPUPWINDOW      = 0x80880000

WS_EX_TOPMOST             = 0x00000008
WS_EX_TRANSPARENT         = 0x00000020
WS_EX_MDICHILD            = 0x00000040
WS_EX_TOOLWINDOW          = 0x00000080
WS_EX_CLIENTEDGE          = 0x00000200
WS_EX_APPWINDOW           = 0x00040000
WS_EX_LAYERED             = 0x00080000
WS_EX_NOREDIRECTIONBITMAP = 0x00200000
WS_EX_COMPOSITED          = 0x02000000
WS_EX_NOACTIVATE          = 0x08000000
WS_EX_ACCEPTFILES         = 0x00000010

GCL_STYLE = -26
CS_VREDRAW, CS_HREDRAW, CS_DBLCLKS, CS_PARENTDC = 0x0001, 0x0002, 0x0008, 0x0080

SWP_NOSIZE         = 0x0001
SWP_NOMOVE         = 0x0002
SWP_NOZORDER       = 0x0004
SWP_NOREDRAW       = 0x0008
SWP_NOACTIVATE     = 0x0010
SWP_FRAMECHANGED   = 0x0020
SWP_SHOWWINDOW     = 0x0040
SWP_HIDEWINDOW     = 0x0080
SWP_NOCOPYBITS     = 0x0100
SWP_NOREPOSITION   = 0x0200
SWP_NOSENDCHANGING = 0x0400
SWP_DEFERERASE     = 0x2000
SWP_ASYNCWINDOWPOS = 0x4000

CW_USEDEFAULT = 0x80000000
WA_INACTIVE, WA_ACTIVE, WA_CLICKACTIVE = 0, 1, 2
INPUT_MOUSE, INPUT_KEYBOARD, INPUT_HARDWARE = 0, 1, 2
PM_NOREMOVE, PM_REMOVE, PM_NOYIELD = 0x0000, 0x0001, 0x0002
SIZE_RESTORED, SIZE_MINIMIZED, SIZE_MAXIMIZED, SIZE_MAXSHOW, SIZE_MAXHIDE = range(5)
KEYEVENTF_EXTENDEDKEY, KEYEVENTF_KEYUP, KEYEVENTF_UNICODE, KEYEVENTF_SCANCODE = 0x0001, 0x0002, 0x0004, 0x0008
IDC_ARROW, IDC_SIZENWSE, IDC_SIZENESW, IDC_SIZEWE, IDC_SIZENS, IDC_SIZEALL = 32512, 32642, 32643, 32644, 32645, 32646
IDS_APP_TITLE = 103
IDI_GDICAPTURINGANIMAGE = 107
IDC_GDICAPTURINGANIMAGE = 109
IDI_SMALL = 108
MAX_LOADSTRING = 100
MB_OK = 0x00000000
HALFTONE = 4
SM_CXSCREEN = 0
SM_CYSCREEN = 1
SRCCOPY = 0x00CC0020
SRCINVERT = 0x00660046
SRCAND = 0x008800C6
BLACKNESS = 0x4200042
CAPTURBLT = 0x40000000
BI_RGB = 0
GMEM_MOVEABLE = 0x0002
GMEM_ZEROINIT = 0x0040
GHND = GMEM_MOVEABLE | GMEM_ZEROINIT
DIB_RGB_COLORS = 0
GENERIC_WRITE = 0x40000000
CREATE_ALWAYS = 2
FILE_ATTRIBUTE_NORMAL = 0x00000080
RDW_INVALIDATE = 0x0001

IDM_ABOUT = 104
IDD_ABOUTBOX = 103
IDM_EXIT = 105

IDOK = 1
IDCANCEL = 2
